# require 'json'
require 'plist'
require_relative '../project'

module Pod
  class Command
    class Vemars
      class Create < Vemars
        self.summary = 'Create iOS vemars project given name and appKey'
        self.description = <<-DESC
            Create one iOS vemars project given name and appKey
        DESC

        self.arguments = [
          CLAide::Argument.new('NAME', false),
          CLAide::Argument.new('VERSION', false),
        ]

        def self.options
          options = [
            ['--language=LANGUAGE',    'language should be objc or swift'],
            ['--silent',    'do not run pod install'],
            ['--bundle_id=bundle_id',    'the bundle_id from Vemars'],
            ['--com=COM1,COM2', 'Selected components in vemars.'],
            ['--config=CONFIG_PATH', 'config path of vemars.'],
            ['--service_url=SERVICE_URL', 'url of vemars CLI service.'],
            ['--git=GIT_URL', 'git url of demo repo'],
            ['--pod_repo_no_update',    'do not run pod repo update'],
            ['--sources','pod repos sources']
          ]
          options.concat(super.reject { |option, _| option == '--silent' })
        end

        def initialize(argv)
          @name = argv.shift_argument
          @version = argv.shift_argument
          @language = argv.option('language', 'objc')
          git_url = argv.option('git', 'https://github.com/volcengine/ve_Template_iOS.git')
          @silent = argv.flag?('silent', false)
          @repo_no_update = argv.flag?('pod_repo_no_update',false)
          @appkey = argv.option('appkey', '')
          @bundle_id = argv.option('bundle_id', nil)
          @selected_components = argv.option('com', "").split(',')
          @config_json = argv.option('config', '/onekit-config.json') 
          service_url = argv.option('service_url',nil)
          sources = argv.option('sources','https://github.com/volcengine/volcengine-specs.git,https://cdn.cocoapods.org/')
          @project = VemarsProject.new(@appkey, @selected_components, @config_json, @version, @bundle_id, @name, @language, git_url,service_url,sources)
          super
          @additional_args = argv.remainder!
        end

        def validate!
          super
          if @config_json.nil? || @config_json.empty?
            help! 'An config.json for the project is required.'
          end
          if @language != 'objc' && @language != 'swift'
            help! 'language should be either objc or swift'
          end
          unless File.exist? @config_json
            help! "onekit-config.json is not found at #{Dir.pwd}"
          end

          # help! '信息有误' unless @project.validate?
        end

        def run
          puts 'Hi, welcome to vemars!'
          @project.generate
          podInstallIfNeeded
          openXcodeWorkSpaceIfNeeded
        end

        def podInstallIfNeeded
          puts "Current dir: #{Dir.pwd}"
          project_dir = File.join(Dir.pwd, [@project.basicInfo.name, "Project"])
          puts "Pod install directory: #{project_dir}"
          if @silent
            puts "Pod install skipped!"
            puts "You can run Pod install in '#{project_dir}' later!"
            puts "You can run Pod install in '#{project_dir}' later!"
            puts "You can run Pod install in '#{project_dir}' later!"
            return
          end

          Dir.chdir(project_dir) do
            # system('bundle install')
            # system('bundle exec pod install --repo-update')
            if @repo_no_update
              puts "pod install with no repo update"
              system('pod install --no-repo-update')
            else
              system('pod install --repo-update')
            end
          end
        end

        def openXcodeWorkSpaceIfNeeded
          workspace_dir = File.join(Dir.pwd, [@project.basicInfo.name, "Project","'#{@project.basicInfo.name}'.xcworkspace"])
          system("open -a /Applications/Xcode.app '#{workspace_dir}'")
        end
      end
    end
  end
end