module Pod
  class PodfileTemplate

    attr_reader :componentsList
    attr_reader :sources
    attr_reader :baseline_version

    IOS_VERSION = '9.0'.freeze

    def initialize(baseline_version, componentsList, sources)
      @baseline_version = baseline_version
      @componentsList = componentsList
      @sources = sources
    end

    def to_dsl
      <<-SPEC

 #{platform}

#plugin 'cocoapods-vemars'
install! 'cocoapods', :deterministic_uuids => false

#{source_template}

#{releasePod}
#{target}
#{post_install_setup}
      SPEC
    end

    def platform
      <<-SPEC
platform :ios, #{IOS_VERSION}
inhibit_all_warnings!
      SPEC
    end

    def source_template
      source_urls = sources.split(",")
      "#{source_urls.map{ |url| "source '#{url}'"}.join("\n")}"
    end

    def releasePod
      <<-SPEC
      
def release_pod
  #baseline version #{baseline_version}
  #{@componentsList.map { |component|
        result = "\tpod '#{component.name}'"
        if !component.version.nil? 
          result = result + ", '#{component.version}'"
        end
        
        if !component.subspecs.nil?
          result = result + ", :subspecs => #{component.subspecs}"
        end
        result
  }.join("\n")}
end
      SPEC
    end

    def target
      <<-SPEC
target 'Template_InHouse' do
  release_pod
  pod 'App/Debug', :path => './../DevPods/', :inhibitat_warnings => false
end
      SPEC
    end

    def post_install_setup
      <<-SPEC
def update_deployment_config(config = nil)
  return if config.nil?
  config.build_settings['ENABLE_BITCODE'] = 'NO'
  if config.build_settings['IPHONEOS_DEPLOYMENT_TARGET'].to_f < 12.0
    config.build_settings['IPHONEOS_DEPLOYMENT_TARGET'] = '12.0'
  end
end

post_install do |installer|
  installer.pods_project.build_configurations.each do |config|
    update_deployment_config(config)
  end
  
  installer.pods_project.targets.each do |target|
    target.build_configurations.each do |config|
      update_deployment_config(config)
    end
  end
  ## for generate_multiple_pod_projects = true
  installer.generated_projects.each do |project|
    project.build_configurations.each do |config|
      update_deployment_config(config)
    end
    
    project.targets.each do |target|
      target.build_configurations.each do |config|
        update_deployment_config(config)
      end
    end
  end

end
      SPEC
    end
  end

end
