module Pod
  class ProjectRenamer
    attr_reader :oldname
    attr_reader :newname

    attr_writer :processedPaths

    def initialize(oldname, newname)
      @oldname = oldname
      @newname = newname
      @processedPaths = []
    end

    def execute
      path = Dir.pwd
      puts "#{path}"
      if validatePath(path)
        enumeratePath(path)
      else
        help! "Xcode project or workspace with name: #{@oldname} is not found in current path."
      end
    end

    private def validatePath(path)
      projectPath = path + "/#{oldname}.xcodeproj"
      workspace = path + "/#{oldname}.xcworkspace"
      isValid = File.exist?(projectPath) || File.exist?(workspace)
      return isValid
    end

    private def enumeratePath(path)
      Dir.each_child(path) do |child|
        childPath = path + "/#{child}"
        if !@processedPaths.include?(childPath) && !shouldSkip(childPath)
          processPath(childPath)
        end
      end
    end

    private def processPath(path)
      # puts "Processing: #{path}"
      if File.directory?(path)
        enumeratePath(path)
      elsif File.exist?(path)
        updateContentsOfFile(path)
      end

      renameItem(path)
      @processedPaths.push(path)
    end

    private def shouldSkip(element)
      if element.include?(".DS_Store") ||
        element.include?("Carthage") ||
        element.include?("Pods") ||
        element.include?("fastlane") ||
        element.include?("build")
        return true
      end

      extension = File.extname(element)
      if extension.include?("appiconset") ||
        extension.include?("json") ||
        extension.include?("png") ||
        extension.include?("xcuserstate")
        return true
      end
      return  false
    end

    private def updateContentsOfFile(path)
      text = File.read(path)
      text = text.gsub(oldname, newname)
      File.open(path, "w") { |file| file.puts text }
    end

    private def renameItem(path)
      oldItemName = File.basename(path)
      if oldItemName.include?(oldname)
        newItemName = oldItemName.gsub(oldname, newname)
        directoryPath = File.dirname(path)
        newPath = directoryPath + "/#{newItemName}"
        File.rename(path, newPath)
        # puts "--- Renamed: #{oldItemName} -> #{newItemName}"
      end
    end
  end
end